library benchmarks.src.compiler.compiler_benchmark;

import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import 'package:angular2/src/dom/browser_adapter.dart' show BrowserDomAdapter;
import 'package:angular2/src/facade/lang.dart' show isBlank, Type;
import 'package:angular2/src/facade/browser.dart' show document;
import 'package:angular2/src/facade/collection.dart' show MapWrapper;
import 'package:angular2/src/core/compiler/directive_metadata.dart'
    show DirectiveMetadata;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show NativeShadowDomStrategy;
import 'package:angular2/change_detection.dart'
    show Parser, Lexer, ProtoRecordRange, dynamicChangeDetection;
import 'package:angular2/src/core/compiler/compiler.dart'
    show Compiler, CompilerCache;
import 'package:angular2/src/core/compiler/directive_metadata_reader.dart'
    show DirectiveMetadataReader;
import 'package:angular2/src/core/annotations/annotations.dart' show Component;
import 'package:angular2/src/core/annotations/annotations.dart' show Decorator;
import 'package:angular2/src/core/annotations/template.dart' show Template;
import 'package:angular2/src/core/compiler/template_loader.dart'
    show TemplateLoader;
import 'package:angular2/src/core/compiler/template_resolver.dart'
    show TemplateResolver;
import 'package:angular2/src/core/compiler/url_resolver.dart' show UrlResolver;
import 'package:angular2/src/core/compiler/style_url_resolver.dart'
    show StyleUrlResolver;
import 'package:angular2/src/core/compiler/component_url_mapper.dart'
    show ComponentUrlMapper;
import 'package:angular2/src/core/compiler/css_processor.dart'
    show CssProcessor;
import 'package:angular2/src/reflection/reflection.dart' show reflector;
import 'package:angular2/src/test_lib/benchmark_util.dart'
    show getIntParameter, bindAction;

setupReflector() {
  reflector.registerType(BenchmarkComponent, {
    "factory": () {
      return new BenchmarkComponent();
    },
    "parameters": [],
    "annotations": [new Component()]
  });
  reflector.registerType(Dir0, {
    "factory": () {
      return new Dir0();
    },
    "parameters": [],
    "annotations": [new Decorator(selector: '[dir0]', bind: {'prop': 'attr0'})]
  });
  reflector.registerType(Dir1, {
    "factory": (dir0) {
      return new Dir1(dir0);
    },
    "parameters": [[Dir0]],
    "annotations": [new Decorator(selector: '[dir1]', bind: {'prop': 'attr1'})]
  });
  reflector.registerType(Dir2, {
    "factory": (dir1) {
      return new Dir2(dir1);
    },
    "parameters": [[Dir1]],
    "annotations": [new Decorator(selector: '[dir2]', bind: {'prop': 'attr2'})]
  });
  reflector.registerType(Dir3, {
    "factory": (dir2) {
      return new Dir3(dir2);
    },
    "parameters": [[Dir2]],
    "annotations": [new Decorator(selector: '[dir3]', bind: {'prop': 'attr3'})]
  });
  reflector.registerType(Dir4, {
    "factory": (dir3) {
      return new Dir4(dir3);
    },
    "parameters": [[Dir3]],
    "annotations": [new Decorator(selector: '[dir4]', bind: {'prop': 'attr4'})]
  });
  reflector.registerGetters({
    "inter0": (a) {
      return a.inter0;
    },
    "inter1": (a) {
      return a.inter1;
    },
    "inter2": (a) {
      return a.inter2;
    },
    "inter3": (a) {
      return a.inter3;
    },
    "inter4": (a) {
      return a.inter4;
    },
    "value0": (a) {
      return a.value0;
    },
    "value1": (a) {
      return a.value1;
    },
    "value2": (a) {
      return a.value2;
    },
    "value3": (a) {
      return a.value3;
    },
    "value4": (a) {
      return a.value4;
    },
    "prop": (a) {
      return a.prop;
    }
  });
  reflector.registerSetters({
    "inter0": (a, v) {
      return a.inter0 = v;
    },
    "inter1": (a, v) {
      return a.inter1 = v;
    },
    "inter2": (a, v) {
      return a.inter2 = v;
    },
    "inter3": (a, v) {
      return a.inter3 = v;
    },
    "inter4": (a, v) {
      return a.inter4 = v;
    },
    "value0": (a, v) {
      return a.value0 = v;
    },
    "value1": (a, v) {
      return a.value1 = v;
    },
    "value2": (a, v) {
      return a.value2 = v;
    },
    "value3": (a, v) {
      return a.value3 = v;
    },
    "value4": (a, v) {
      return a.value4 = v;
    },
    "attr0": (a, v) {
      return a.attr0 = v;
    },
    "attr1": (a, v) {
      return a.attr1 = v;
    },
    "attr2": (a, v) {
      return a.attr2 = v;
    },
    "attr3": (a, v) {
      return a.attr3 = v;
    },
    "attr4": (a, v) {
      return a.attr4 = v;
    },
    "prop": (a, v) {
      return a.prop = v;
    }
  });
}
main() {
  BrowserDomAdapter.makeCurrent();
  var count = getIntParameter('elements');
  setupReflector();
  var reader = new DirectiveMetadataReader();
  var cache = new CompilerCache();
  var templateResolver = new FakeTemplateResolver();
  var urlResolver = new UrlResolver();
  var styleUrlResolver = new StyleUrlResolver(urlResolver);
  var compiler = new Compiler(dynamicChangeDetection,
      new TemplateLoader(null, urlResolver), reader, new Parser(new Lexer()),
      cache, new NativeShadowDomStrategy(styleUrlResolver), templateResolver,
      new ComponentUrlMapper(), urlResolver, new CssProcessor(null));
  var templateNoBindings = createTemplateHtml('templateNoBindings', count);
  var templateWithBindings = createTemplateHtml('templateWithBindings', count);
  compileNoBindings() {
    templateResolver.setTemplateHtml(templateNoBindings);
    cache.clear();
    compiler.compile(BenchmarkComponent);
  }
  compileWithBindings() {
    templateResolver.setTemplateHtml(templateWithBindings);
    cache.clear();
    compiler.compile(BenchmarkComponent);
  }
  bindAction('#compileNoBindings', compileNoBindings);
  bindAction('#compileWithBindings', compileWithBindings);
}
createTemplateHtml(templateId, repeatCount) {
  var template = DOM.querySelectorAll(document, '''#${templateId}''')[0];
  var content = DOM.getInnerHTML(template);
  var result = '';
  for (var i = 0; i < repeatCount; i++) {
    result += content;
  }
  return result;
}
@Decorator(selector: '[dir0]', bind: const {'prop': 'attr0'})
class Dir0 {}
@Decorator(selector: '[dir1]', bind: const {'prop': 'attr1'})
class Dir1 {
  Dir1(Dir0 dir0) {}
}
@Decorator(selector: '[dir2]', bind: const {'prop': 'attr2'})
class Dir2 {
  Dir2(Dir1 dir1) {}
}
@Decorator(selector: '[dir3]', bind: const {'prop': 'attr3'})
class Dir3 {
  Dir3(Dir2 dir2) {}
}
@Decorator(selector: '[dir4]', bind: const {'prop': 'attr4'})
class Dir4 {
  Dir4(Dir3 dir3) {}
}
@Component()
class BenchmarkComponent {}
class FakeTemplateResolver extends TemplateResolver {
  Template _template;
  FakeTemplateResolver() : super() {}
  setTemplateHtml(String html) {
    this._template =
        new Template(inline: html, directives: [Dir0, Dir1, Dir2, Dir3, Dir4]);
  }
  Template resolve(Type component) {
    return this._template;
  }
}
