library benchmarks.src.naive_infinite_scroll.cells;

import 'package:angular2/src/facade/lang.dart' show int;
import 'package:angular2/src/reflection/reflection.dart' show reflector;
import 'package:angular2/src/test_lib/benchmark_util.dart'
    show getIntParameter, bindAction;
import 'package:angular2/angular2.dart'
    show bootstrap, Component, Viewport, Template, ViewContainer, Compiler;
import 'package:angular2/src/facade/async.dart' show PromiseWrapper;
import 'package:angular2/src/facade/collection.dart'
    show ListWrapper, MapWrapper;
import './common.dart'
    show Company, Opportunity, Offering, Account, CustomDate, STATUS_LIST;
import 'package:angular2/directives.dart' show For;

class HasStyle {
  Map style;
  HasStyle() {
    this.style = MapWrapper.create();
  }
  set width(w) {
    MapWrapper.set(this.style, 'width', w);
  }
}
class CompanyNameComponent extends HasStyle {
  Company company;
}
class OpportunityNameComponent extends HasStyle {
  Opportunity opportunity;
}
class OfferingNameComponent extends HasStyle {
  Offering offering;
}
class Stage {
  String name;
  bool isDisabled;
  Map style;
  Function apply;
}
class StageButtonsComponent extends HasStyle {
  Offering _offering;
  List<Stage> stages;
  Offering get offering {
    return this._offering;
  }
  set offering(Offering offering) {
    this._offering = offering;
    this._computeStageButtons();
  }
  setStage(Stage stage) {
    this._offering.status = stage.name;
    this._computeStageButtons();
  }
  _computeStageButtons() {
    var disabled = true;
    this.stages = ListWrapper.clone(STATUS_LIST.map((status) {
      var isCurrent = this._offering.status == status;
      var stage = new Stage();
      stage.name = status;
      stage.isDisabled = disabled;
      var stageStyle = MapWrapper.create();
      MapWrapper.set(stageStyle, 'background-color',
          disabled ? '#DDD' : isCurrent ? '#DDF' : '#FDD');
      stage.style = stageStyle;
      if (isCurrent) {
        disabled = false;
      }
      return stage;
    }));
  }
}
class AccountCellComponent extends HasStyle {
  Account account;
}
class FormattedCellComponent extends HasStyle {
  String formattedValue;
  set value(value) {
    if (value is CustomDate) {
      this.formattedValue = '''${value.month}/${value.day}/${value.year}''';
    } else {
      this.formattedValue = value.toString();
    }
  }
}
setupReflectorForCells() {
  reflector.registerType(CompanyNameComponent, {
    'factory': () {
      return new CompanyNameComponent();
    },
    'parameters': [],
    'annotations': [
      new Component(
          selector: 'company-name',
          bind: {'width': 'cell-width', 'company': 'company'}),
      new Template(
          directives: [],
          inline: '''<div [style]=\"style\">{{company.name}}</div>''')
    ]
  });
  reflector.registerType(OpportunityNameComponent, {
    'factory': () {
      return new OpportunityNameComponent();
    },
    'parameters': [],
    'annotations': [
      new Component(
          selector: 'opportunity-name',
          bind: {'width': 'cell-width', 'opportunity': 'opportunity'}),
      new Template(
          directives: [],
          inline: '''<div [style]=\"style\">{{opportunity.name}}</div>''')
    ]
  });
  reflector.registerType(OfferingNameComponent, {
    'factory': () {
      return new OfferingNameComponent();
    },
    'parameters': [],
    'annotations': [
      new Component(
          selector: 'offering-name',
          bind: {'width': 'cell-width', 'offering': 'offering'}),
      new Template(
          directives: [],
          inline: '''<div [style]=\"style\">{{offering.name}}</div>''')
    ]
  });
  reflector.registerType(StageButtonsComponent, {
    'factory': () {
      return new StageButtonsComponent();
    },
    'parameters': [],
    'annotations': [
      new Component(
          selector: 'stage-buttons',
          bind: {'width': 'cell-width', 'offering': 'offering'}),
      new Template(directives: [For], inline: '''
            <div [style]=\"style\">
                <button template=\"for #stage of stages\"
                        [disabled]=\"stage.isDisabled\"
                        [style]=\"stage.style\"
                        on-click=\"setStage(stage)\">
                  {{stage.name}}
                </button>
            </div>''')
    ]
  });
  reflector.registerType(AccountCellComponent, {
    'factory': () {
      return new AccountCellComponent();
    },
    'parameters': [],
    'annotations': [
      new Component(
          selector: 'account-cell',
          bind: {'width': 'cell-width', 'account': 'account'}),
      new Template(directives: [], inline: '''
            <div [style]=\"style\">
              <a href=\"/account/{{account.accountId}}\">
                {{account.accountId}}
              </a>
            </div>''')
    ]
  });
  reflector.registerType(FormattedCellComponent, {
    'factory': () {
      return new FormattedCellComponent();
    },
    'parameters': [],
    'annotations': [
      new Component(
          selector: 'formatted-cell',
          bind: {'width': 'cell-width', 'value': 'value'}),
      new Template(
          directives: [],
          inline: '''<div [style]=\"style\">{{formattedValue}}</div>''')
    ]
  });
}
