library benchmarks.src.tree.tree_benchmark;

import 'package:angular2/change_detection.dart'
    show Parser, Lexer, ChangeDetector, ChangeDetection, jitChangeDetection;
import 'package:angular2/src/core/exception_handler.dart' show ExceptionHandler;
import 'package:angular2/angular2.dart'
    show
        bootstrap,
        Component,
        Viewport,
        Template,
        ViewContainer,
        Compiler,
        NgElement,
        Decorator;
import 'package:angular2/src/core/compiler/compiler.dart' show CompilerCache;
import 'package:angular2/src/core/compiler/directive_metadata_reader.dart'
    show DirectiveMetadataReader;
import 'package:angular2/src/core/compiler/template_loader.dart'
    show TemplateLoader;
import 'package:angular2/src/core/compiler/template_resolver.dart'
    show TemplateResolver;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show
        ShadowDomStrategy,
        NativeShadowDomStrategy,
        EmulatedUnscopedShadowDomStrategy;
import 'package:angular2/src/core/compiler/shadow_dom_emulation/content_tag.dart'
    show Content;
import 'package:angular2/src/core/compiler/shadow_dom_emulation/light_dom.dart'
    show DestinationLightDom;
import 'package:angular2/src/core/life_cycle/life_cycle.dart' show LifeCycle;
import 'package:angular2/src/core/compiler/url_resolver.dart' show UrlResolver;
import 'package:angular2/src/core/compiler/style_url_resolver.dart'
    show StyleUrlResolver;
import 'package:angular2/src/core/compiler/component_url_mapper.dart'
    show ComponentUrlMapper;
import 'package:angular2/src/core/compiler/style_inliner.dart'
    show StyleInliner;
import 'package:angular2/src/core/compiler/css_processor.dart'
    show CssProcessor;
import 'package:angular2/src/core/compiler/private_component_loader.dart'
    show PrivateComponentLoader;
import 'package:angular2/src/reflection/reflection.dart' show reflector;
import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import 'package:angular2/src/facade/lang.dart' show isPresent;
import 'package:angular2/src/facade/browser.dart' show window, document, gc;
import 'package:angular2/src/test_lib/benchmark_util.dart'
    show getIntParameter, bindAction;
import 'package:angular2/src/core/compiler/xhr/xhr.dart' show XHR;
import 'package:angular2/src/core/compiler/xhr/xhr_impl.dart' show XHRImpl;
import 'package:angular2/directives.dart' show If;
import 'package:angular2/src/dom/browser_adapter.dart' show BrowserDomAdapter;
import 'package:angular2/src/core/events/event_manager.dart' show EventManager;

setupReflector() {
  reflector.registerType(AppComponent, {
    'factory': () {
      return new AppComponent();
    },
    'parameters': [],
    'annotations': [
      new Component(selector: 'app'),
      new Template(
          directives: [TreeComponent],
          inline: '''<tree [data]=\'initData\'></tree>''')
    ]
  });
  reflector.registerType(TreeComponent, {
    'factory': () {
      return new TreeComponent();
    },
    'parameters': [],
    'annotations': [
      new Component(selector: 'tree', bind: {'data': 'data'}),
      new Template(
          directives: [TreeComponent, If],
          inline: '''<span> {{data.value}} <span template=\'if data.right != null\'><tree [data]=\'data.right\'></tree></span><span template=\'if data.left != null\'><tree [data]=\'data.left\'></tree></span></span>''')
    ]
  });
  reflector.registerType(If, {
    'factory': (vp) {
      return new If(vp);
    },
    'parameters': [[ViewContainer]],
    'annotations': [new Viewport(selector: '[if]', bind: {'condition': 'if'})]
  });
  reflector.registerType(Compiler, {
    'factory': (cd, templateLoader, reader, parser, compilerCache, strategy,
        tplResolver, cmpUrlMapper, urlResolver, cssProcessor) {
      return new Compiler(cd, templateLoader, reader, parser, compilerCache,
          strategy, tplResolver, cmpUrlMapper, urlResolver, cssProcessor);
    },
    'parameters': [
      [ChangeDetection],
      [TemplateLoader],
      [DirectiveMetadataReader],
      [Parser],
      [CompilerCache],
      [ShadowDomStrategy],
      [TemplateResolver],
      [ComponentUrlMapper],
      [UrlResolver],
      [CssProcessor]
    ],
    'annotations': []
  });
  reflector.registerType(CompilerCache, {
    'factory': () {
      return new CompilerCache();
    },
    'parameters': [],
    'annotations': []
  });
  reflector.registerType(Parser, {
    'factory': (lexer) {
      return new Parser(lexer);
    },
    'parameters': [[Lexer]],
    'annotations': []
  });
  reflector.registerType(TemplateLoader, {
    'factory': (xhr, urlResolver) {
      return new TemplateLoader(xhr, urlResolver);
    },
    'parameters': [[XHR], [UrlResolver]],
    'annotations': []
  });
  reflector.registerType(TemplateResolver, {
    'factory': () {
      return new TemplateResolver();
    },
    'parameters': [],
    'annotations': []
  });
  reflector.registerType(XHR, {
    'factory': () {
      return new XHRImpl();
    },
    'parameters': [],
    'annotations': []
  });
  reflector.registerType(DirectiveMetadataReader, {
    'factory': () {
      return new DirectiveMetadataReader();
    },
    'parameters': [],
    'annotations': []
  });
  reflector.registerType(ShadowDomStrategy, {
    "factory": (strategy) {
      return strategy;
    },
    "parameters": [[NativeShadowDomStrategy]],
    "annotations": []
  });
  reflector.registerType(NativeShadowDomStrategy, {
    "factory": (styleUrlResolver) {
      return new NativeShadowDomStrategy(styleUrlResolver);
    },
    "parameters": [[StyleUrlResolver]],
    "annotations": []
  });
  reflector.registerType(EmulatedUnscopedShadowDomStrategy, {
    "factory": (styleUrlResolver) {
      return new EmulatedUnscopedShadowDomStrategy(styleUrlResolver, null);
    },
    "parameters": [[StyleUrlResolver]],
    "annotations": []
  });
  reflector.registerType(StyleUrlResolver, {
    "factory": (urlResolver) {
      return new StyleUrlResolver(urlResolver);
    },
    "parameters": [[UrlResolver]],
    "annotations": []
  });
  reflector.registerType(Content, {
    "factory": (lightDom, el) {
      return new Content(lightDom, el);
    },
    "parameters": [[DestinationLightDom], [NgElement]],
    "annotations": [new Decorator(selector: '[content]')]
  });
  reflector.registerType(UrlResolver, {
    "factory": () {
      return new UrlResolver();
    },
    "parameters": [],
    "annotations": []
  });
  reflector.registerType(Lexer, {
    'factory': () {
      return new Lexer();
    },
    'parameters': [],
    'annotations': []
  });
  reflector.registerType(ExceptionHandler, {
    "factory": () {
      return new ExceptionHandler();
    },
    "parameters": [],
    "annotations": []
  });
  reflector.registerType(LifeCycle, {
    "factory": (exHandler, cd) {
      return new LifeCycle(exHandler, cd);
    },
    "parameters": [[ExceptionHandler], [ChangeDetector]],
    "annotations": []
  });
  reflector.registerType(ComponentUrlMapper, {
    "factory": () {
      return new ComponentUrlMapper();
    },
    "parameters": [],
    "annotations": []
  });
  reflector.registerType(StyleInliner, {
    "factory": (xhr, styleUrlResolver, urlResolver) {
      return new StyleInliner(xhr, styleUrlResolver, urlResolver);
    },
    "parameters": [[XHR], [StyleUrlResolver], [UrlResolver]],
    "annotations": []
  });
  reflector.registerType(CssProcessor, {
    "factory": () {
      return new CssProcessor(null);
    },
    "parameters": [],
    "annotations": []
  });
  reflector.registerType(EventManager, {
    "factory": () {
      return new EventManager([], null);
    },
    "parameters": [],
    "annotations": []
  });
  reflector.registerType(PrivateComponentLoader, {
    "factory": (compiler, strategy, eventMgr, reader) {
      return new PrivateComponentLoader(compiler, strategy, eventMgr, reader);
    },
    "parameters": [
      [Compiler],
      [ShadowDomStrategy],
      [EventManager],
      [DirectiveMetadataReader]
    ],
    "annotations": []
  });
  reflector.registerGetters({
    'value': (a) {
      return a.value;
    },
    'left': (a) {
      return a.left;
    },
    'right': (a) {
      return a.right;
    },
    'initData': (a) {
      return a.initData;
    },
    'data': (a) {
      return a.data;
    },
    'condition': (a) {
      return a.condition;
    }
  });
  reflector.registerSetters({
    'value': (a, v) {
      return a.value = v;
    },
    'left': (a, v) {
      return a.left = v;
    },
    'right': (a, v) {
      return a.right = v;
    },
    'initData': (a, v) {
      return a.initData = v;
    },
    'data': (a, v) {
      return a.data = v;
    },
    'condition': (a, v) {
      return a.condition = v;
    },
    'if': (a, v) {
      return a['if'] = v;
    }
  });
}
var BASELINE_TREE_TEMPLATE;
var BASELINE_IF_TEMPLATE;
main() {
  BrowserDomAdapter.makeCurrent();
  var maxDepth = getIntParameter('depth');
  setupReflector();
  BASELINE_TREE_TEMPLATE = DOM.createTemplate(
      '<span>_<template class="ng-binding"></template><template class="ng-binding"></template></span>');
  BASELINE_IF_TEMPLATE =
      DOM.createTemplate('<span template="if"><tree></tree></span>');
  var app;
  var lifeCycle;
  var baselineRootTreeComponent;
  var count = 0;
  ng2DestroyDom() {
    app.initData = new TreeNode('', null, null);
    lifeCycle.tick();
  }
  profile(create, destroy, name) {
    return () {
      window.console.profile(name + ' w GC');
      var duration = 0;
      var count = 0;
      while (count++ < 150) {
        gc();
        var start = window.performance.now();
        create();
        duration += window.performance.now() - start;
        destroy();
      }
      window.console.profileEnd(name + ' w GC');
      window.console.log(
          '''Iterations: ${count}; time: ${duration / count} ms / iteration''');
      window.console.profile(name + ' w/o GC');
      duration = 0;
      count = 0;
      while (count++ < 150) {
        var start = window.performance.now();
        create();
        duration += window.performance.now() - start;
        destroy();
      }
      window.console.profileEnd(name + ' w/o GC');
      window.console.log(
          '''Iterations: ${count}; time: ${duration / count} ms / iteration''');
    };
  }
  ng2CreateDom() {
    var values = count++ % 2 == 0
        ? ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '*']
        : ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', '-'];
    app.initData = buildTree(maxDepth, values, 0);
    lifeCycle.tick();
  }
  noop() {}
  initNg2() {
    bootstrap(AppComponent).then((injector) {
      lifeCycle = injector.get(LifeCycle);
      app = injector.get(AppComponent);
      bindAction('#ng2DestroyDom', ng2DestroyDom);
      bindAction('#ng2CreateDom', ng2CreateDom);
      bindAction(
          '#ng2UpdateDomProfile', profile(ng2CreateDom, noop, 'ng2-update'));
      bindAction('#ng2CreateDomProfile',
          profile(ng2CreateDom, ng2DestroyDom, 'ng2-create'));
    });
  }
  baselineDestroyDom() {
    baselineRootTreeComponent.update(new TreeNode('', null, null));
  }
  baselineCreateDom() {
    var values = count++ % 2 == 0
        ? ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '*']
        : ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', '-'];
    baselineRootTreeComponent.update(buildTree(maxDepth, values, 0));
  }
  initBaseline() {
    var tree = DOM.createElement('tree');
    DOM.appendChild(DOM.querySelector(document, 'baseline'), tree);
    baselineRootTreeComponent = new BaseLineTreeComponent(tree);
    bindAction('#baselineDestroyDom', baselineDestroyDom);
    bindAction('#baselineCreateDom', baselineCreateDom);
    bindAction('#baselineUpdateDomProfile',
        profile(baselineCreateDom, noop, 'baseline-update'));
    bindAction('#baselineCreateDomProfile',
        profile(baselineCreateDom, baselineDestroyDom, 'baseline-create'));
  }
  initNg2();
  initBaseline();
}
class TreeNode {
  String value;
  TreeNode left;
  TreeNode right;
  TreeNode(value, left, right) {
    this.value = value;
    this.left = left;
    this.right = right;
  }
}
buildTree(maxDepth, values, curDepth) {
  if (identical(maxDepth, curDepth)) return new TreeNode('', null, null);
  return new TreeNode(values[curDepth],
      buildTree(maxDepth, values, curDepth + 1),
      buildTree(maxDepth, values, curDepth + 1));
}
class BaseLineTreeComponent {
  var element;
  BaseLineInterpolation value;
  BaseLineIf left;
  BaseLineIf right;
  BaseLineTreeComponent(element) {
    this.element = element;
    var clone = DOM.clone(BASELINE_TREE_TEMPLATE.content.firstChild);
    var shadowRoot = this.element.createShadowRoot();
    DOM.appendChild(shadowRoot, clone);
    var child = clone.firstChild;
    this.value = new BaseLineInterpolation(child);
    child = DOM.nextSibling(child);
    this.left = new BaseLineIf(child);
    child = DOM.nextSibling(child);
    this.right = new BaseLineIf(child);
  }
  update(TreeNode value) {
    this.value.update(value.value);
    this.left.update(value.left);
    this.right.update(value.right);
  }
}
class BaseLineInterpolation {
  String value;
  var textNode;
  BaseLineInterpolation(textNode) {
    this.value = null;
    this.textNode = textNode;
  }
  update(String value) {
    if (!identical(this.value, value)) {
      this.value = value;
      DOM.setText(this.textNode, value + ' ');
    }
  }
}
class BaseLineIf {
  bool condition;
  BaseLineTreeComponent component;
  var anchor;
  BaseLineIf(anchor) {
    this.anchor = anchor;
    this.condition = false;
    this.component = null;
  }
  update(TreeNode value) {
    var newCondition = isPresent(value);
    if (!identical(this.condition, newCondition)) {
      this.condition = newCondition;
      if (isPresent(this.component)) {
        DOM.remove(this.component.element);
        this.component = null;
      }
      if (this.condition) {
        var element = DOM.firstChild(DOM.clone(BASELINE_IF_TEMPLATE).content);
        this.anchor.parentNode.insertBefore(
            element, DOM.nextSibling(this.anchor));
        this.component = new BaseLineTreeComponent(DOM.firstChild(element));
      }
    }
    if (isPresent(this.component)) {
      this.component.update(value);
    }
  }
}
class AppComponent {
  TreeNode initData;
  AppComponent() {
    this.initData = new TreeNode('', null, null);
  }
}
class TreeComponent {
  TreeNode data;
}
